
package regression;

import regression.data.Grammar;
import regression.data.Sample;
import regression.evolution.Population;

import java.io.IOException;


/**
 * Symbolic regression tool.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Tool {

	public static void main(String[] args) throws IOException {
		new Tool().run(args);
	}

	private void run(String[] args) throws IOException {

		if ( args.length != 2 ) {
			throw new IllegalArgumentException(
					"usage: "+getClass().getName()+" <sample> <grammar>");
		}

		final Sample sample=new Sample(args[0]);
		final Grammar grammar=new Grammar(args[1]);

		final Population population=new Population(sample, grammar);

		System.out.println("---------------------------------------- Trace --");

		while ( !population.terminated() ) {
			System.out.println(population);
			population.evolve();
		}

		System.out.println("------------------------------------- Solution --");
		System.out.println(population);
	}
}
