
package regression.data;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


/**
 * Data sample.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Sample {

	private final double[] x;
	private final double[] y;


	public Sample(final String file) throws IOException {

		final Tokenizer tokenizer=new Tokenizer(file);

		final List x=new ArrayList();
		final List y=new ArrayList();

		for (String[] tokens; (tokens=tokenizer.readTokens()) != null; ) {

			if ( tokens.length != 2 ) {
				throw new RuntimeException(
						"syntax error at line "+tokenizer.getLineNumber());
			}

			x.add(new Double(tokens[0]));
			y.add(new Double(tokens[1]));
		}

		this.x=new double[x.size()];
		this.y=new double[y.size()];

		for (int n=0; n < this.x.length; n++) {
			this.x[n]=((Double)x.get(n)).doubleValue();
			this.y[n]=((Double)y.get(n)).doubleValue();
		}
	}

	public double rmse(final Function function) {

		if ( function == null ) {
			throw new IllegalArgumentException("null function");
		}

		double error=0.0;

		for (int n=0; n < x.length; n++) {
			final double delta=function.evaluate(x[n])-y[n];
			error+=delta*delta;
		}

		return Double.isNaN(error) || Double.isInfinite(error) ? Double.MAX_VALUE
				: x.length > 0 ? Math.sqrt(error/x.length)
				: 0.0;
	}
}
