
package regression.data;

import java.util.ArrayList;
import java.util.List;


/**
 * Grammatical sentence.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Sentence {

	private final Grammar grammar;
	private final List tokens;


	public Sentence(final Grammar grammar) {

		if ( grammar == null ) {
			throw new IllegalArgumentException("null grammar");
		}

		this.grammar=grammar;
		this.tokens=new ArrayList();

		tokens.add(grammar.start());
	}

	private String token(final int index) {
		return (String)tokens.get(index);
	}


	public int length() {
		return tokens.size();
	}

	public int options(final int index) {
		return grammar.options((String)tokens.get(index));
	}

	public void expand(final int index, final int option) {

		final String[] expansion=grammar.expand(token(index), option);

		tokens.remove(index);

		for (int n=0; n < expansion.length; n++) {
			tokens.add(index+n, expansion[n]);
		}
	}

	public String toString() {

		final StringBuffer buffer=new StringBuffer();

		for (int n=0; n < length(); ++n) {

			final String token=token(n);

			if ( grammar.isTerminal(token) ) {
				buffer.append(token);
			} else {
				buffer.append('<');
				buffer.append(token);
				buffer.append('>');
			}
		}

		return buffer.toString();
	}
}
