
package regression.data;

import java.io.*;
import java.util.StringTokenizer;


/**
 * File tokenizer.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
final class Tokenizer {

	private final LineNumberReader reader;


	public Tokenizer(final String file) throws FileNotFoundException {

		if ( file == null ) {
			throw new IllegalArgumentException("null data file");
		}

		this.reader=new LineNumberReader(new FileReader(file));
		this.reader.setLineNumber(1);
	}


	public int getLineNumber() {
		return reader.getLineNumber();
	}

	public String[] readTokens() throws IOException {

		for (String line; (line=readLine()) != null;) {
			if ( line.length() == 0 || line.startsWith("#") ) {
				// Skip blank lines and comments
			} else {
				return tokenize(line);
			}
		}

		return null;
	}


	private String readLine() throws IOException {

		final String line=reader.readLine();

		return line != null ? line.trim() : null;
	}

	private String[] tokenize(final String line) {

		if ( line == null ) {
			throw new IllegalArgumentException("null line");
		}

		final StringTokenizer tokenizer=new StringTokenizer(line);
		final String[] tokens=new String[tokenizer.countTokens()];

		for (int n=0; n < tokens.length; n++) {
			tokens[n]=tokenizer.nextToken();
		}

		return tokens;
	}
}
