
package regression.evolution;

/**
 * Standardized fitness value.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Fitness {

	static public final Fitness BEST=new Fitness(0.0);
	static public final Fitness WORST=new Fitness(Double.MAX_VALUE);

	static private final double tolerance=1e-3;

	private final double value;


	public Fitness(final double value) {

		if ( value < 0 ) {
			throw new IllegalArgumentException("negative fitness value");
		}

		this.value=value;
	}


	public boolean isOptimal() {
		return value == 0.0 || value < tolerance;
	}

	public boolean isBetterThan(final Fitness reference) {

		if ( reference == null ) {
			throw new IllegalArgumentException("null reference");
		}

		return value < reference.value;
	}

	public String toString() {
		return String.valueOf(value);
	}
}
