
package regression.evolution;

import regression.data.Function;
import regression.data.Grammar;
import regression.data.Sentence;

import java.util.Random;


/**
 * Genotype of evolvable functions.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Genotype {

	static private final int lengthRange=10;

	private final byte[] bases;
	private final Random random;


	public Genotype() {

		this.random=new Random();
		this.bases=new byte[random.nextInt(lengthRange-1)+1];

		random.nextBytes(bases);
	}

	private Genotype(final byte[] bases) {

		if ( bases == null ) {
			throw new IllegalArgumentException("null bases");
		}

		this.bases=bases;
		this.random=new Random();
	}


	public Genotype recombine(final Genotype peer) {

		final int head=(this.bases.length > 1) ?
				random.nextInt(this.bases.length-1)+1 : 1;
		final int tail=(peer.bases.length > 1) ?
				random.nextInt(peer.bases.length-1)+1 : 1;

		final byte[] bases=new byte[head+tail];
		System.arraycopy(this.bases, 0, bases, 0, head);
		System.arraycopy(peer.bases, peer.bases.length-tail, bases, head, tail);

		return new Genotype(bases);
	}

	public Genotype mutate() {

		final int locus=random.nextInt(this.bases.length);

		final byte[] bases=(byte[])this.bases.clone();
		bases[locus]=(byte)(random.nextInt()&0xFF);

		return new Genotype(bases);
	}

	public Genotype replicate() {
		return new Genotype((byte[])bases.clone());
	}


	public Phenotype translate(final Grammar grammar) {

		final Sentence sentence=new Sentence(grammar);

		for (int index=0, locus=0; index < sentence.length();) {

			final int options=sentence.options(index);

			if ( options == 0 ) {
				
				++index;

			} else if ( options < 255 ) {

				if ( locus < bases.length ) {
					sentence.expand(index, transcribe(locus++, options));

				} else {
					return new Phenotype();
				}

			} else {
				throw new RuntimeException(
						"more than 256 productions for a single symbol");
			}
		}

		return new Phenotype(new Function(sentence.toString()));
	}

	public int transcribe(final int locus, final int options) {
		return (bases[locus]&0xFF)%options;
	}

}
