
package regression.evolution;

import regression.data.Grammar;
import regression.data.Sample;

import java.util.Random;


/**
 * Evolvable function.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Individual {

	static private final double recombinationRate=0.7;
	static private final double mutationRate=0.1;


	private final Sample sample;
	private final Grammar grammar;
	private final Random random;

	private final Genotype genotype;
	private final Phenotype phenotype;
	private final Fitness fitness;


	public Individual(final Sample sample, final Grammar grammar) {
		this(new Genotype(), sample, grammar);
	}

	private Individual(final Genotype genotype,
			final Sample sample, final Grammar grammar) {

		if ( genotype == null ) {
			throw new IllegalArgumentException("null genotype");
		}

		if ( sample == null ) {
			throw new IllegalArgumentException("null sample");
		}

		if ( grammar == null ) {
			throw new IllegalArgumentException("null grammar");
		}

		this.sample=sample;
		this.grammar=grammar;
		this.random=new Random();

		this.genotype=genotype;
		this.phenotype=genotype.translate(grammar);
		this.fitness=phenotype.evaluate(sample);
	}


	public Individual reproduce(final Individual peer) {

		final double event=random.nextDouble();
		final Genotype genotype;

		if ( event < recombinationRate ) {
			genotype=this.genotype.recombine(peer.genotype);
		} else if ( event < mutationRate+recombinationRate ) {
			genotype=this.genotype.mutate();
		} else {
			genotype=this.genotype.replicate();
		}

		return new Individual(genotype, sample, grammar);
	}


	public boolean isOptimal() {
		return fitness.isOptimal();
	}

	public boolean isBetterThan(final Individual reference) {

		if ( reference == null ) {
			throw new IllegalArgumentException("null reference individual");
		}

		return fitness.isBetterThan(reference.fitness);
	}

	public String toString() {
		return fitness+"\t"+phenotype;
	}
}
