
package regression.evolution;

import regression.data.Function;
import regression.data.Sample;


/**
 * Phenotype of evolvable functions.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Phenotype {

	private final Function function;


	public Phenotype() {
		this.function=null;
	}

	public Phenotype(final Function function) {

		if ( function == null ) {
			throw new IllegalArgumentException("null function");
		}

		this.function=function;
	}


	public Fitness evaluate(final Sample sample) {

		if ( sample == null ) {
			throw new IllegalArgumentException("null sample");
		}

		return function != null ?
				new Fitness(sample.rmse(function)) : Fitness.WORST;
	}

	public String toString() {
		return function != null ? function.toString() : "<null>";
	}
}
