
package regression.evolution;

import regression.data.Grammar;
import regression.data.Sample;

import java.util.Random;


/**
 * Population of evolvable functions.
 *
 * @author  Copyright &copy; 2004 Alessandro Bollini. All rights reserved.
 */
public final class Population {

	static private final int populationSize=1000;
	static private final int tournamentSize=3;

	static private final int iterationLimit=50;


	private int iteration;
	private Individual solution;
	private final Random random;

	private final Individual[] individuals;


	public Population(final Sample sample, final Grammar grammar) {

		if ( sample == null ) {
			throw new IllegalArgumentException("null sample");
		}

		if ( grammar == null ) {
			throw new IllegalArgumentException("null grammar");
		}

		this.iteration=0;
		this.solution=new Individual(sample, grammar);
		this.random=new Random();

		this.individuals=new Individual[populationSize];

		for (int n=0; n < individuals.length; ++n) {

			this.individuals[n]=new Individual(sample, grammar);

			if ( this.individuals[n].isBetterThan(this.solution) ) {
				this.solution=this.individuals[n];
			}
		}
	}


	public boolean terminated() {
		return solution.isOptimal() || iteration >= iterationLimit;
	}

	public void evolve() {

		for (int n=0; n < individuals.length; ++n) {

			final Individual parent=select();
			final Individual peer=select();
			final Individual offspring=parent.reproduce(peer);

			final int target=random.nextInt(individuals.length);
			individuals[target]=offspring;

			if ( offspring.isBetterThan(solution) ) {
				solution=offspring;
			}
		}

		++iteration;
	}

	private Individual select() {

		Individual winner=sample();

		for (int n=1; n < tournamentSize; ++n) {

			final Individual challenger=sample();

			if ( challenger.isBetterThan(winner) ) {
				winner=challenger;
			}
		}

		return winner;
	}

	private Individual sample() {
		return individuals[random.nextInt(individuals.length)];
	}

	public String toString() {
		return solution.toString();
	}
}
